// Core logic for managing filters
import React from "react";
import { createContext, useCallback, useContext, useState, } from "react";
const ListingContext = createContext({});
export function ListingFilterProvider({ children, }) {
    const [filterStates, setFilterStates] = useState({});
    const onFilterChange = useCallback((name, value) => setFilterStates((p) => (Object.assign(Object.assign({}, p), { [name]: value }))), [setFilterStates]);
    const getFilterState = useCallback((n) => {
        const item = filterStates[n];
        if (item)
            return item === null || item === void 0 ? void 0 : item.value;
        else
            return null;
    }, [filterStates]);
    const getCurrentFilterQuery = useCallback(() => ({
        and: Object.values(filterStates).reduce((p, c) => [
            ...p,
            c.queryFunc(c.value),
        ], []),
    }), [filterStates]);
    const removeFilter = useCallback((name) => {
        setFilterStates((pS) => {
            var _a;
            return (_a = Object === null || Object === void 0 ? void 0 : Object.keys(pS)) === null || _a === void 0 ? void 0 : _a.filter((n) => n !== name).reduce((p, c) => {
                return Object.assign(Object.assign({}, p), { [c]: pS[c] });
            }, {});
        });
    }, [setFilterStates]);
    return (React.createElement(ListingContext.Provider, { value: {
            getCurrentFilterQuery,
            onFilterChange,
            getFilterState,
            removeFilter,
            filterStates,
        } }, children));
}
export function useFilterController({ queryFunc, name, }) {
    const { onFilterChange, getFilterState, removeFilter } = useListingFilterProvider();
    return {
        onChange: (v) => onFilterChange(name, { value: v, queryFunc }),
        resetFilter: () => removeFilter(name),
        value: getFilterState(name),
    };
}
export function useListingFilterProvider() {
    return useContext(ListingContext);
}
